## input: vector of GO terms
## output: heatmap semantic similarity scores
library(tools)
library(GOSemSim)
library(ComplexHeatmap)
library(circlize)
library('BiocParallel')  
library(data.table)
library(tidyverse)

## GO data from mouse org database for biological processes
## Can do for mouse phenotype and molecular processes
mmGO = godata('org.Mm.eg.db', ont="BP") 
plot_dir = ("consensus/promoters/")
## combine = 'BMA' returns a single value estimating how similar the 2 sets are, you can change it with min,max,avg ... (see link above)

GOsimilarity <- function(fileList, file_to_compare_to){
    files =list.files(fileList, pattern= "", full.names=T) # create list of files in directory
    files = as.list(files)
    list = lapply(files, function(x) read.table(x, header=TRUE, sep="\t", quote = "") %>% as.data.table()) # read in all files
    ## this keeps only the significant GO terms
    significant_go_terms=lapply(list,function(x) x=x[p.adjust<=0.01])
    listVector = lapply(significant_go_terms, function(x) dplyr::pull(x, ID))
    file_input = read.table(file_to_compare_to, sep="\t", header=TRUE, quote="") %>% as.data.frame()
    file_sig_go = dplyr::pull(file_input[file_input$p.adjust <= 0.01,], ID)

    ## This extracts the file names for all the files in the list (all embryonic stages)
    file_names=as.character(list.files(fileList,recursive =F,full.names = F)) ## directory with all embryonic stages
    file_names=gsub("\\..*","",file_names) ## Removes everything except 'E*.5'
    file_names=gsub("[^0-9]","", file_names) ## Removes the 'E' so that it's a number
    names(listVector)=file_names ## Add new names to list

    ## Check if input file name is less than or equal to any in listVector
    ## This will avoid running comparisons that have already been run
    ## Eg. E10.5 vs E11.5 is the same as E11.5 vs E10.5
    file_to_compare_to_name = basename(file_path_sans_ext(file_to_compare_to)) ## Name of file to compare to
    if(grepl('^E', file_to_compare_to_name) == TRUE){ ## If file begins with E (this is so that it doesn't stall with the dunnart files)
        file_to_compare_to_name=gsub("\\..*","",file_to_compare_to_name) ## Removes everything except 'E*.5'
        file_to_compare_to_name=gsub("[^0-9]","",file_to_compare_to_name) ## Removes the 'E' so that it's a number
        listVectorSubset = listVector[names(listVector) > file_to_compare_to_name] ## Keep only vectors in the list that are greater than the file being compared to
                                                                                ## Eg. this means that when comparing E12 to the list then it will only compare to 13,14 & 15
    } else 
    listVectorSubset = listVector

    ## Compare the similarity between GO lists
    ## This will parallelise the lapply function 
    comparisons = bplapply(listVectorSubset,function(x) mgoSim(file_sig_go,x, semData=mmGO,measure="Wang", combine='BMA')) 
    return(comparisons)
}

## This command checks the time it takes to run
## One comparison is about 95 seconds
system.time(mgoSim(listVector[[1]], listVector[[2]],semData=mmGO, measure="Wang", combine='BMA'))

register(SerialParam())


## Promoters
plot_dir = ("consensus/promoters/")

dunnartvsmouse = GOsimilarity(fileList = "consensus/promoters/mouse_GO", 
file_to_compare_to = "consensus/promoters/dunnart_GO/promoter_mm10GOenrich") #dunnart versus all mouse stages

e10vs = GOsimilarity(fileList = "consensus/promoters/mouse_GO", 
file_to_compare_to = "consensus/promoters/mouse_GO/E10.5_promoter_mm10GOenrich") 

e11vs = GOsimilarity(fileList = "consensus/promoters/mouse_GO", 
file_to_compare_to = "consensus/promoters/mouse_GO/E11.5_promoter_mm10GOenrich")

e12vs = GOsimilarity(fileList = "consensus/promoters/mouse_GO", 
file_to_compare_to = "consensus/promoters/mouse_GO/E12.5_promoter_mm10GOenrich") 

e13vs = GOsimilarity(fileList = "consensus/promoters/mouse_GO", 
file_to_compare_to = "consensus/promoters/mouse_GO/E13.5_promoter_mm10GOenrich") 

e14vs = GOsimilarity(fileList = "consensus/promoters/mouse_GO", 
file_to_compare_to = "consensus/promoters/mouse_GO/E14.5_promoter_mm10GOenrich") 


## Enhancers
plot_dir = ("consensus/enhancers/")

dunnartvsmouse = GOsimilarity(fileList = "consensus/enhancers/mouse_GO", 
file_to_compare_to = "consensus/enhancers/dunnart_GO/enhancer_mm10GOenrich_subsample.txt")

e10vs = GOsimilarity(fileList = "consensus/enhancers/mouse_GO", 
file_to_compare_to = "consensus/enhancers/mouse_GO/E10.5_enhancer_mm10GOenrich") #dunnart versus all mouse stages

e11vs = GOsimilarity(fileList = "consensus/enhancers/mouse_GO", 
file_to_compare_to = "consensus/enhancers/mouse_GO/E11.5_enhancer_mm10GOenrich") #dunnart versus all mouse stages

e12vs = GOsimilarity(fileList = "consensus/enhancers/mouse_GO", 
file_to_compare_to = "consensus/enhancers/mouse_GO/E12.5_enhancer_mm10GOenrich") #dunnart versus all mouse stages

e13vs = GOsimilarity(fileList = "consensus/enhancers/mouse_GO", 
file_to_compare_to = "consensus/enhancers/mouse_GO/E13.5_enhancer_mm10GOenrich") #dunnart versus all mouse stages

e14vs = GOsimilarity(fileList = "consensus/enhancers/mouse_GO", 
file_to_compare_to = "consensus/enhancers/mouse_GO/E14.5_enhancer_mm10GOenrich") #dunnart versus all mouse stages


vector_of_scores <- c(
    dunnartvsmouse$'10', dunnartvsmouse$'11', dunnartvsmouse$'12', dunnartvsmouse$'13', dunnartvsmouse$'14',dunnartvsmouse$'15',
    e10vs$'11',e10vs$'12',e10vs$'13',e10vs$'14',e10vs$'15',
    e11vs$'12',e11vs$'13',e11vs$'14',e11vs$'15',
    e12vs$'13',e12vs$'14',e12vs$'15',
    e13vs$'14',e13vs$'15',
    e14vs$'15'

)
my_matrix <- matrix(0,7,7) ## creates a n x n square 0 matrix

rownames(my_matrix) = c('Dunnart','E10','E11','E12','E13','E14','E15')
colnames(my_matrix) = c('Dunnart','E10','E11','E12','E13','E14','E15')

my_matrix[ col(my_matrix) < row(my_matrix) ] <- vector_of_scores
my_matrix <- my_matrix + t(my_matrix)
diag(my_matrix) <- 1
my_matrix

pdf(paste(plot_dir,'enhancer_GO_semantic_scores.pdf',sep=''),width=10,height = 7)
Heatmap(my_matrix, name = "score", col=c('#0868ac', '#43a2ca','#7bccc4', '#bae4bc','#f0f9e8'))
dev.off()

pdf(paste(plot_dir,'promoter_GO_semantic_scores.pdf',sep=''),width=10,height = 7)
Heatmap(my_matrix, name = "score", col=c('#810f7c', '#8856a7','#8c96c6', '#b3cde3','#edf8fb'), heatmap_legend_param = list(
        title = "GO similarity score", at = c(0.7,0.8,0.9, 1)))
dev.off()


