# Author: Laura Cook
# Purpose:
#       1. Generate txdb for dunnart annotation
#       2. Annotate dunnart peaks with ChIPseeker
#       3. Perform GO and KEGG enrichment analysis and plot


# Load libraries
library(GenomicFeatures)
library(GenomicRanges)
library(rtracklayer)
library(AnnotationForge)
library(clusterProfiler)
library(ChIPseeker)
library(ggpubr)
library(AnnotationDbi)
library(AnnotationHub)
library(org.Mm.eg.db)
library(plyr)
library(tidyverse)
library(RColorBrewer)
library(ggplot2)
library(data.table)

## Make txdb for dunnart annotation file
setwd("/data/projects/punim0586/lecook/chipseq-pipeline/dunnart/results/macs2")
smiCra_txdb <- makeTxDbFromGFF("Scras_dunnart_assem1.0_pb-ont-illsr_flyeassem_red-rd-scfitr2_pil2xwgs2_60chr2.gff")

## geneID conversion tables
setwd("/data/projects/punim0586/lecook/chipseq-pipeline/cross_species/peakAnnotation/")
df2 <- read.table("biomart_ensembl.txt", header=TRUE, sep="\t") ## conversion table for devil ENSEMBL to mouse ENSEMBL
df3 <- read.table("bioDBnet_refseq2ensembl_H3K27ac.txt", header=TRUE, sep="\t") ## convertsion table for devil refseq to devil ENSEMBL

#################################################### DUNNART ANNOTATION ##########################################################
setwd("/data/projects/punim0586/lecook/chipseq-pipeline/dunnart/results/macs2")
# Background for calculating enrichment
# These are mouse ENSEMBL IDs for genes where there is an orthologous gene in the Tas devil 
bg=fread("go_background_orthologousENSEMBL_biomart.txt", header=FALSE, sep="\t", quote = "", na.strings=c("", "NA"))
bg=unlist(bg, recursive=TRUE, use.names = FALSE)
## Anotate peak files
annotatePeaks <- function(peak, outFile, outFile1, outFile2, annoBar, distanceTSS, heatmapTSS, GOenrich, kegg){
  
  # Annotate peak file based on dunnart GFF
  peakAnno <- annotatePeak(peak, tssRegion = c(-3000, 3000), TxDb = smiCra_txdb)

  # Write annotation to file
  write.table(peakAnno, outFile, sep="\t", quote=F, row.names=F)
  
  peakAnnoDF <- as.data.frame(peakAnno, row.names = NULL)
  # # Convert refseq IDs and geneIDs to devil ensembl IDs

  peakAnnoDF$ensemblgeneID <- df2$Gene.stable.ID[match(unlist(peakAnnoDF$geneId), df2$Gene.name)]
  peakAnnoDF$ensemblgeneID = replace(peakAnnoDF$ensemblgeneID,is.na(peakAnnoDF$ensemblgeneID),"-")
  peakAnnoDF$transcriptIdAltered <- gsub("\\..*","", peakAnnoDF$transcriptId)
  peakAnnoDF$refseqID <- df3$Ensembl.Gene.ID[match(unlist(peakAnnoDF$transcriptIdAltered), df3$RefSeq.mRNA.Accession)]
  peakAnnoDF$refseqID = replace(peakAnnoDF$refseqID,is.na(peakAnnoDF$refseqID),"-")
  peakAnnoDF$combined <- ifelse(peakAnnoDF$refseqID == "-", peakAnnoDF$ensemblgeneID, peakAnnoDF$refseqID)
  peakAnnoDF$combined[peakAnnoDF$combined == as.character("-")] <- NA

  peakAnnoDF <- peakAnnoDF[!is.na(peakAnnoDF$combined),]
  write.table(peakAnnoDF, outFile2, sep="\t", quote=F, row.names=F)
  # # Convert devil ensembl to mouse ensembl
  peakAnnoDF$mouseensembl <- df2$Mouse.gene.stable.ID[match(unlist(peakAnnoDF$combined), df2$Gene.stable.ID)]
  # # Write annotation with converted IDs 
  peakAnnoDF$mouseensembl[peakAnnoDF$mouseensembl == as.character("")] <- NA
  peakAnnoDF <- peakAnnoDF[!is.na(peakAnnoDF$mouseensembl),]
  write.table(peakAnnoDF, outFile1, sep="\t", quote=F, row.names=F)

  # # GO enrichment analysis
  GO <- enrichGO(gene = peakAnnoDF$mouseensembl,
                  keyType = "ENSEMBL",
                  OrgDb = org.Mm.eg.db,
                  ont = "BP",
                  universe = bg,
                  pAdjustMethod = "fdr",
                  qvalueCutoff = 0.01,
                  readable = TRUE)
  write.table(GO, GOenrich, sep="\t", quote=F, row.names=F)
  
  # convert mouse ensembl to entrezID
  suppressWarnings(gene.df <- bitr(geneID=peakAnnoDF$mouseensembl, fromType="ENSEMBL", toType="ENTREZID", OrgDb="org.Mm.eg.db" ))
  
  # ## kegg analysis
  kk <- enrichKEGG(gene = gene.df$ENTREZID, organism = "mmu", pAdjustMethod = "fdr",
                  qvalueCutoff = 0.01, universe = )
  write.table(kk, kegg, sep="\t", quote=F, row.names=F)
}

### Consensus peaks
annotatePeaks(peak = "dunnart_enhancers.narrowPeak", outFile = "dunnart_enhancer_annotation.txt", 
annoBar = "dunnart_enhancer_annoBar.pdf", distanceTSS = "dunnart_enhancer_distanceTSS.pdf", 
heatmapTSS = "dunnart_enhancer_heatmapTSS.pdf", outFile1 = "dunnart_enhancer_annotationConvertedIDs.txt", 
GOenrich = "enhancer_mm10GOenrich.txt", kegg="enhancer_KEGG.txt", outFile2 = "dunnart_enhancer_annotationConvertedIDs_t.devil.txt")

annotatePeaks(peak = "dunnart_promoters.narrowPeak", outFile = "dunnart_promoter_annotation.txt", 
  annoBar = "dunnart_promoter_annoBar.pdf", distanceTSS = "dunnart_promoter_distanceTSS.pdf", 
  heatmapTSS = "dunnart_promoter_heatmapTSS.pdf", outFile1 = "dunnart_promoter_annotationConvertedIDs.txt" , 
  GOenrich = "promoter_mm10GOenrich.txt", kegg="promoter_KEGG.txt", outFile2 = "dunnart_promoter_annotationConvertedIDs_t.devil.txt")

annotatePeaks(peak = "cluster1_promoters.narrowPeak", outFile = "cluster1_promoter_annotation.txt", 
  annoBar = "cluster1_promoter_annoBar.pdf", distanceTSS = "cluster1_promoter_distanceTSS.pdf", 
  heatmapTSS = "cluster1_promoter_heatmapTSS.pdf", outFile1 = "cluster1_promoter_annotationConvertedIDs.txt" , 
  GOenrich = "cluster1_mm10GOenrich.txt", kegg="cluster1_KEGG.txt", outFile2 = "cluster1_annotationConvertedIDs_t.devil.txt")

promoter = fread("cluster1_promoter_annotationConvertedIDs.txt")
enhancer = fread("dunnart_enhancer_annotationConvertedIDs.txt")
data = list(promoter, enhancer)
genes = lapply(data, function(i) as.data.frame(i)$mouseensembl)
genes = lapply(genes, function(i) unique(i))
names(genes) = c("promoter-associated peaks", "enhancer-associated peaks")
go_cluster <- setReadable(
      compareCluster(
        geneCluster = genes, 
        fun = enrichGO, 
        ont="BP",
        universe = bg, 
        keyType="ENSEMBL", 
        pvalueCutoff = 0.001, 
        OrgDb = org.Mm.eg.db),
      OrgDb = org.Mm.eg.db, 
      keyType="ENSEMBL")

## Dotplot
pdf("dunnart_go_cluster.pdf", width = 11, height = 9)
print(dotplot(go_cluster, showCategory = 10) + 
scale_color_viridis() +
theme(axis.text.x = element_text(angle = 45, hjust=1)))
dev.off()

write.table(go_cluster@compareClusterResult, file = "dunnart_peak_mm10_enrichGO_compareCluster_results.txt", sep="\t", quote=F, row.names=F)
suppressWarnings(genes_ent <- lapply(genes, function(x) bitr(geneID=x, fromType="ENSEMBL", toType="ENTREZID", OrgDb="org.Mm.eg.db" )))
genes_ent = lapply(genes_ent, function(x) x$ENTREZID)
suppressWarnings(bg_ent <- bitr(geneID=bg, fromType="ENSEMBL", toType="ENTREZID", OrgDb="org.Mm.eg.db" ))
bg_ent = bg_ent$ENTREZID

kegg_cluster <- 
  compareCluster(
  geneCluster = genes_ent, 
  fun = 'enrichKEGG', 
  universe = bg_ent, 
  pvalueCutoff = 0.001, 
  organism = 'mmu')
    
pdf("dunnart_kegg_cluster.pdf", width = 8, height = 6)
print(dotplot(kegg_cluster, showCategory = 10) + 
scale_color_viridis() + 
theme(axis.text.x = element_text(angle = 45, hjust=1)))
dev.off()

split =  split(as.data.frame(go_cluster@compareClusterResult), f = factor(go_cluster@compareClusterResult$Cluster))
go_terms = lapply(split, function(x) x$ID)
pdf("dunnart_simplifyGO.pdf", width = 10, height = 14)
    simplifyGOFromMultipleLists(go_terms, db = org.Mm.eg.db,  measure = "Wang", method = "binary_cut")
    dev.off()  

intersect = intersect(go_terms[[1]], go_terms[[2]])
union = union(go_terms[[1]], go_terms[[2]])
promoter_setdiff = setdiff(go_terms[[1]], go_terms[[2]])

promoter_setdiff_terms = go_cluster@compareClusterResult[go_cluster@compareClusterResult[,"ID"] %in% promoter_setdiff,]
enhancer_setdiff_terms = go_cluster@compareClusterResult[go_cluster@compareClusterResult[,"ID"] %in% enhancer_setdiff,]
